#! /bin/bash

#
# Compile and boot with tests for extended assignment 1.
# The correct result of each test is signaled by
#
# Test passed...
#

fail() {
	rm -f test.log
	echo
	echo "Failure: $1"
	exit 1
}

# Don't output command executed by make unless run with -v
if [ "$1" == "-v" ] ; then
	SILENT_MAKE=""
else
	SILENT_MAKE="--silent"
fi

emake() {
	echo "Running make $SILENT_MAKE $@"
	make $SILENT_MAKE "$@"
}

for TEST in \
    tests/as1e/condvar1/test.c \
    tests/as1e/condvar2/test.c \
    tests/as1e/condvar3/test.c \
    tests/as1e/rwlock1/test.c \
    tests/as1e/rwlock2/test.c \
    tests/as1e/rwlock3/test.c \
    tests/as1e/rwlock4/test.c \
    tests/as1e/semaphore1/test.c \
    tests/as1e/semaphore2/test.c \
    tests/as1e/semaphore3/test.c \
    tests/as1e/semaphore4/test.c \
    tests/as1e/spinlock1/test.c \
    tests/as1e/spinlock2/test.c \
    ; do
	emake distclean || fail "Cleanup before compilation"
	emake "KERNEL_TEST=$TEST" || fail "Compilation"
	msim | tee test.log || fail "Execution"
	grep '^Test passed\.\.\.$' test.log > /dev/null || fail "Test $TEST"
	rm -f test.log
	emake distclean || fail "Cleanup after compilation"
done

echo
echo "All tests passed..."
